@@ -423,6 +423,30 @@ err_out:
 	return -1;
 }
 
+/* Return 0 if fname/value valid, -1 otherwise */
+int get_devtree_value(const char *fname, unsigned long long *value)
+{
+	FILE *file;
+	char buf[MAXBYTES];
+	int n = -1;
+
+	if ((file = fopen(fname, "r"))) {
+		n = fread(buf, 1, MAXBYTES, file);
+		fclose(file);
+	}
+
+	if (n == sizeof(uint32_t))
+		*value = ((uint32_t *)buf)[0];
+	else if (n == sizeof(uint64_t))
+		*value = ((uint64_t *)buf)[0];
+	else {
+		fprintf(stderr, "%s node has invalid size: %d\n", fname, n);
+		return -1;
+	}
+
+	return 0;
+}
+
 /* Get devtree details and create exclude_range array
  * Also create usablemem_ranges for KEXEC_ON_CRASH
  */
@@ -736,7 +760,6 @@ static int get_devtree_details(unsigned long kexec_flags)
 					!= 1) {
 				perror(fname);
 				goto error_openfile;
-				return -1;
 			}
 			memset(fname, 0, sizeof(fname));
 			sprintf(fname, "%s%s%s",
@@ -943,3 +966,7 @@ void arch_update_purgatory(struct kexec_info *UNUSED(info))
 {
 }
 
+int arch_do_exclude_segment(struct kexec_info *UNUSED(info), struct kexec_segment *UNUSED(segment))
+{
+	return 0;
+}
